package combinatorParsing

/** An object encpsulating basic character constants */
object CharArrayReader {
  final val NewLineCh = '\012'
  final val EofCh = '\032'
}

/** A character array reader allows to read an array of character while
 *  maintaining position information.
 */
class CharArrayReader(buf: Array[char]) {
  import CharArrayReader._

  /** The last character read */
  var ch: char = NewLineCh

  /** The line of last character read (starts at position 1)
   */
  var line = 0

  /** The column of last character read (starts at position 1)
   */
  var column: int = _

  private final val CR = '\015'

  private var bp = 0

  /** Set <code>ch</code> to next input character; update position. */
  def next {
    if (ch == NewLineCh) {
      line = line + 1
      column = 1
    } else if (ch != EofCh) {
      column = column + 1
    }
    if (bp == buf.length) {
      ch = EofCh
    } else {
      ch = buf(bp)
      bp = bp + 1
      if (ch == CR && bp < buf.length && buf(bp) == NewLineCh) next
    }
  }
}
