package combinatorParsing

/** This trait defines the standard token classes
 *  Token classes have two curried argument lists.
 *  The first argument list gives information specific to each token.
 *  The second argument list gives the token's position.
 *  That way, positions do not form part of patterns over tokens.
 *  Instead, use `t.pos' to get the position of token `t'.
 */
trait Tokens {

  /** The root class of tokens */
  class Token(val pos: Position)

  /** The class of keyword tokens */
  case class KW(chars: String)(pos: Position) extends Token(pos)

  /** The class of numeric literal tokens */
  case class NumericLit(chars: String)(pos: Position) extends Token(pos)

  /** The class of string literal tokens */
  case class StringLit(chars: String)(pos: Position) extends Token(pos)

  /** The class of identifier tokens */
  case class Identifier(chars: String)(pos: Position) extends Token(pos)

  /** A class of error tokens. Error tokens are used to communicate
   *  errors detected during lexical analysis */
  case class ErrorToken(msg: String)(pos: Position) extends Token(pos)

  /** A class for end-of-file tokens */
  case class EOF()(pos: Position) extends Token(pos)
}
